//Object.cpp: member functions for the Object.h
//Daniel CopyRight Reserved December, 17, 2001


//system includes
#include <math.h>
#include <stdio.h>


//custom includes
#include "Object.h" //header file
#include "Timer.h"
#include "Random.h"
#include "Font.h"


extern CTimer Timer;
extern CRandom Random;
extern CWorld2D GameWorld;
extern RECT ScreenRECT;


//for debug use----------------------------------------------
extern debugID;
extern LPDIRECTDRAWSURFACE lpPriSurface;
extern CDdFont SurfaceFnt;



//*************************************************************************************
//CStillObject O********************************************************************
//*************************************************************************************

//Constructor -------------------------------------------------------------------------
CStillObject::CStillObject()
{
	Sprite=NULL;
}

//إ ----------------------------------------------------------------------------
BOOL CStillObject::Create(CSprite *sprite, int x, int y, int frameno, int frameperiod)
{

	if(!sprite) return FALSE;
	Sprite=sprite; 
	FrameNum=Sprite->TotalFrames();
	FrameW=Sprite->FrameWidth();
	FrameH=Sprite->FrameHeight();

	curX=x; curY=y; curFrameNo=frameno; FramePeriod=frameperiod;
	LastFrameTime=Timer.time();

	return TRUE;
}


//N{b Frame e Y Surface W----------------------------------------
void CStillObject::Draw(LPDIRECTDRAWSURFACE lpSurface)
{
	POINT scrPoint;

	if(Timer.timeElapsed(LastFrameTime,FramePeriod)) 
	{		
		curFrameNo=(curFrameNo+1>=FrameNum)?0:(curFrameNo+1);
	}	

	scrPoint.x=GameWorld.WorldToScreenX(curX);
	scrPoint.y=GameWorld.WorldToScreenY(curY);

    if(PtInRect(&ScreenRECT, scrPoint))
	  Sprite->Draw2Surface(lpSurface, scrPoint.x-FrameW/2, 
								scrPoint.y-FrameH, curFrameNo);	
}

//return curX-------------------------------------------------------------------
int CStillObject::PosX() 
{
	return curX;
}

//return curY-------------------------------------------------------------------
int CStillObject::PosY()
{
	return curY;
}










//***************************************************************************************
//CWiseObject O***********************************************************************
//*************************************************************************************

//Constructor------------------------------------------------------------------
CWiseObject::CWiseObject()
{
	for(int i=center; i<=corner315; i++) Sprite[i]=NULL;

	MinSpeed=50; MaxSpeed=1500;
	curDir=center;
}


BOOL CWiseObject::Create(CSprite *sprite[corner315+1], int x, int y, int frameno, int frameperiod,
					int Vx, int Vy) //]wm		
{
	for(int i=center; i<=corner315; i++) 
	if(sprite[i]) Sprite[i]=sprite[i];
	else return FALSE;

	FrameNum=Sprite[center]->TotalFrames();
	FrameW=Sprite[center]->FrameWidth();
	FrameH=Sprite[center]->FrameHeight();

	curX=x; curY=y; curFrameNo=frameno; FramePeriod=frameperiod;
	LastFrameTime=Timer.time();

	XSpeed=(Vx==0)?Random.number(5,20):Vx; 
	YSpeed=(Vy==0)?Random.number(5,20):Vy;
	CurSpeed=int(_hypot(double (XSpeed), double(YSpeed)));
	LastMoveTime=Timer.time();
	
	RefreshDirection(XSpeed, YSpeed);

	return TRUE;
}


void CWiseObject::Draw(LPDIRECTDRAWSURFACE lpSurface)
{

	POINT scrPoint;

	if(Timer.timeElapsed(LastFrameTime,FramePeriod)) 
	{		
		curFrameNo=(curFrameNo+1>=FrameNum)?0:(curFrameNo+1);
	}	

	scrPoint.x=GameWorld.WorldToScreenX(curX);
	scrPoint.y=GameWorld.WorldToScreenY(curY);

    if(PtInRect(&ScreenRECT, scrPoint))
	  Sprite[curDir]->Draw2Surface(lpSurface, scrPoint.x-FrameW/2, 
								scrPoint.y-FrameH, curFrameNo);	
}


//----------------------------------------------------------------------------
void CWiseObject::Move()
{
   int nxtX,nxtY,xc,yc,dt=Timer.time()-LastMoveTime;
   double l;
   //short row, col;

   LastMoveTime=Timer.time();

   /*if(!GameWorld.InsideWorld(curX,curY,&row,&col)) 
   {	
	   //for debug use-----------------------------------------------
	   char s[10];
	   sprintf(s,"OutSide World: Row:%-3d, Col:%-3d" , GameWorld.RowOfXY(curY), GameWorld.ColOfXY(curX));
	   SurfaceFnt.Draw_Text(s,5,100,RGB(255,0,0),lpPriSurface);	  
	   
	   Sleep(5000);

	   //GameWorld.GetRandomPosInWorld(&curX,&curY);
   }*/

   nxtX=curX+(XSpeed*dt/1000);
   nxtY=curY+(YSpeed*dt/1000);
   
   if(GameWorld.HitWorldEdge(curX,curY,&nxtX,&nxtY,&xc, &yc))
   {	   
	   curX=xc; curY=yc;
	   l=_hypot(double(nxtX-xc),double(nxtY-yc));

	   XSpeed=int((nxtX-xc)*CurSpeed/l); 	   
	   YSpeed=int((nxtY-yc)*CurSpeed/l); 
	   
	   //קKJûΤB
	   XSpeed=(XSpeed==0)?Random.Sign()*CurSpeed:XSpeed;
	   YSpeed=(YSpeed==0)?Random.Sign()*CurSpeed:YSpeed;

	   RefreshDirection(XSpeed, YSpeed);
   }
   else //no hit worldedge
   {
	   /*if(!GameWorld.InsideWorld(nxtX,nxtY,&row,&col))
	    //StAo]~HDH  GameWorld.HitWorldEdge()  Bug
	   {
		   //for debug use-----------------------------------------------
			char s[10];
			sprintf(s,"OutSide World: Row:%-3d, Col:%-3d" , GameWorld.RowOfXY(curY), GameWorld.ColOfXY(curX));
			SurfaceFnt.Draw_Text(s,5,100,RGB(255,0,0),lpPriSurface);	
			//Sleep(5000);
			//-------------------------------------------------------------		
	   }*/

	   curX=nxtX;
	   curY=nxtY;
   }   
}

//[t------------------------------------------------------------------------------
void CWiseObject::Accelerate(int dv)
{
	int OldSpeed=CurSpeed;	
	
	CurSpeed+=dv;

	if(CurSpeed>MaxSpeed) CurSpeed=MaxSpeed;
	else if(CurSpeed<MinSpeed) CurSpeed=MinSpeed;

	XSpeed=CurSpeed*XSpeed/OldSpeed; 
	YSpeed=CurSpeed*YSpeed/OldSpeed;

	/*//for debug use-----------------------------------------------
	char s[10];
	sprintf(s,"CurSpeed:%-3d, XSp:%-3d, YSp:%-3d", CurSpeed, XSpeed, YSpeed);
	SurfaceFnt.Draw_Text(s,5,100,RGB(255,0,0),lpPriSurface);	
	//-------------------------------------------------------------	*/
}


void CWiseObject::RefreshDirection(int x, int y)
{
	if (x==0)
	{
		if(y==0) curDir=center;
		else if (y>0) curDir=top;
		else curDir=bottom;
	}
	else if(x>0)
	{
		if(y==0) curDir=right;
		else if (y>0) curDir=corner45;
		else curDir=corner315;
	}
	else
	{
		if(y==0) curDir=left;
		else if (y>0) curDir=corner135;
		else curDir=corner225;
	}
}








//*************************************************************************************
//CPlayerObject O***********************************************************************
//*************************************************************************************

//Consturctor------------------------------------------------------------------
CPlayerObject::CPlayerObject() : CWiseObject() //~ CWiseObject  Constructor ҦeAA[Ws lưʧ@C
{
	AutoMove=false;
}


//----------------------------------------------------------------------------
void CPlayerObject::Move()
{
   int nxtX,nxtY,xc,yc,dt=Timer.time()-LastMoveTime;
   double l;
   //short row, col;

   LastMoveTime=Timer.time();   

   nxtX=curX+(XSpeed*dt/1000);
   nxtY=curY+(YSpeed*dt/1000);
   
   if(GameWorld.HitWorldEdge(curX,curY,&nxtX,&nxtY,&xc, &yc))
   {	   
	   curX=xc; curY=yc;
	   l=_hypot(double(nxtX-xc),double(nxtY-yc));

	   XSpeed=int((nxtX-xc)*CurSpeed/l); 	   
	   YSpeed=int((nxtY-yc)*CurSpeed/l); 	
	   
	   //קKJûΤB
	   XSpeed=(XSpeed==0)?Random.Sign()*CurSpeed:XSpeed;
	   YSpeed=(YSpeed==0)?Random.Sign()*CurSpeed:YSpeed;

	   RefreshDirection(XSpeed, YSpeed);
   }
   else //no hit worldedge
   {
	   /*if(!GameWorld.InsideWorld(nxtX,nxtY,&row,&col))
	    //StAo]~HDH  GameWorld.HitWorldEdge()  Bug
	   {
		   //for debug use-----------------------------------------------
			char s[10];
			sprintf(s,"OutSide World: Row:%-3d, Col:%-3d" , GameWorld.RowOfXY(curY), GameWorld.ColOfXY(curX));
			SurfaceFnt.Draw_Text(s,5,100,RGB(255,0,0),lpPriSurface);	
			//Sleep(5000);
			//-------------------------------------------------------------		
	   }*/

	   curX=nxtX;
	   curY=nxtY;
   }       

   GameWorld.ViewX=curX;
   GameWorld.ViewY=curY;
}





//------------------------------------------------------------------------------------
void CPlayerObject::IncMove(int dx, int dy)
{	
	RefreshDirection(dx,dy);

	GameWorld.IncMoveViewPoint(dx, dy);

	curX=GameWorld.ViewX;  
	curY=GameWorld.ViewY;
}


//]wsm----------------------------------------------------------------------------
void CPlayerObject::SetPos(int x, int y)
{
	curX=x;
	curY=y;
}





