//sprite.cpp: member functions for the sprite.h
//Daniel CopyRight Reserved December, 17, 2001


//system includes
#include <ddraw.h>


//custom includes
#include "sprite.h" //header file
#include "ddrawUtil.h"



// Constructor------------------------------------------------------------------------
CSprite::CSprite()
{
	ImgSpritePtr=NULL;
	lpSpriteSurfaces=NULL;
	FrameNum=FrameW=FrameH=0;	
}

//Destructor---------------------------------------------------------------------------
CSprite::~CSprite()
{
	if(lpSpriteSurfaces) delete[] lpSpriteSurfaces;
}


//tm Video Ram (Surface) ŶC-------------------------------------------------------
BOOL CSprite::CreateSurfaces() 
{
	DDSURFACEDESC ddsd; //direct draw surface descriptor 

	//]wz⪺ L index  
	DDCOLORKEY ddck; //direct draw color descriptor
	for (int i=0; i<FrameNum; i++)
	{    
		//إ  surface
		ddsd.dwSize=sizeof(ddsd);
		ddsd.dwFlags=DDSD_CAPS|DDSD_HEIGHT|DDSD_WIDTH;
		//]wª Surface  Flip  Alpha Blend 
		ddsd.ddsCaps.dwCaps= DDSCAPS_OFFSCREENPLAIN;
		//]wSurfacee
		ddsd.dwHeight=FrameH;//SCREEN_HEIGHT; 
		ddsd.dwWidth=FrameW;//SCREEN_WIDTH;	
		
		ddck.dwColorSpaceLowValue= ddck.dwColorSpaceHighValue=TRANSPARENT_COLOR; //one color

		//tm Background Surface Ŷ
		if(FAILED(lpDdObject->CreateSurface(&ddsd,&lpSpriteSurfaces[i],NULL)))
		return FALSE;	

		//]wz
		lpSpriteSurfaces[i]->SetColorKey(DDCKEY_SRCBLT,&ddck);
	}

	return TRUE;
}


BOOL CSprite::CreateSprite(CBmpSpriteFileReader *imgptr, int ImgX, int ImgY,
					  int frames, int framew, int frameh)
{
	if(!imgptr) return FALSE;

	//즳 Surfaces ΪOЪŶ-----------------------------------------
	ReleaseSurfaces();
	//ps Surfaces ƶqMjp---------------------------------
	ImgSpritePtr=imgptr; ImgStartX=ImgX; ImgStartY=ImgY;
	FrameNum=frames; FrameW=framew; FrameH=frameh;
	FrameRect.left=FrameRect.top=0; 
	FrameRect.right=FrameRect.left+FrameW; FrameRect.bottom=FrameRect.top+FrameH;
	//tm Surfaces ƶq
	lpSpriteSurfaces=new LPDIRECTDRAWSURFACE [FrameNum];
	//ھڷsjptm Surfaces Ŷ-----------------------------------------
	if(!CreateSurfaces()) return FALSE;	
	//Nϧεe Surfaces W-----------------------------------------------
	RestoreSurfaces();

	return TRUE;
}


//NY Frame  Surface Blt  Y Surface W-----------------------------------------
void CSprite::Draw2Surface(LPDIRECTDRAWSURFACE lpDestSurface, int x, int y, int FrameNo)
{
	RECT rect;

	rect.left=x; rect.top=y; rect.right=rect.left+FrameW; rect.bottom=rect.top+FrameH;

	lpDestSurface->Blt(&rect,lpSpriteSurfaces[FrameNo],&FrameRect, DDBLT_KEYSRC|DDBLT_WAIT,NULL);		
}

//NY Frame  Surface 50% ze Y Surface W-----------------------------------------
void CSprite::AlphaDraw2Surface(LPDIRECTDRAWSURFACE lpDestSurface, int x, int y, int FrameNo)
{

	CBlendSurface(lpDestSurface, x, y, lpSpriteSurfaces[FrameNo],FrameW, FrameH);


	//U Source P CBlendSurface ()
	/*
	DDSURFACEDESC src_ddsd,dest_ddsd; //direct draw surface descriptor
	BYTE *dest,*src; //destination and source pointers
	
	src_ddsd.dwSize=sizeof(src_ddsd);
	dest_ddsd.dwSize=sizeof(dest_ddsd);

	//ӷMت Surface OаϡAȮɤ\L{sC
	if(FAILED(lpSpriteSurfaces[FrameNo]->Lock(NULL,&src_ddsd, DDLOCK_WAIT,NULL))) return;
	if(FAILED(lpDestSurface->Lock(NULL,&dest_ddsd,DDLOCK_WAIT,NULL))) return;
	
	src=(BYTE*) src_ddsd.lpSurface;
	dest=(BYTE*)dest_ddsd.lpSurface; //destination pointer
	int i, j, cursrcPos=0, curdestPos, ddw=dest_ddsd.lPitch,ddwsrc=src_ddsd.lPitch;


	for(j=y; j<y+FrameH; j++)
	{		
		for(i=x; i<x+FrameW; i++)	
		{
			curdestPos=j*ddw+i;
			if(src[cursrcPos]==TRANSPARENT_COLOR);
			else
			  dest[curdestPos]=AlphaTable[src[cursrcPos]][dest[curdestPos]];
			cursrcPos++;
		}
		cursrcPos=cursrcPos+(ddwsrc-FrameW);
	}

	//w
	lpSpriteSurfaces[FrameNo]->Unlock(NULL);
	lpDestSurface->Unlock(NULL);
	*/
}


//eҦϧθƨ Surfaces W---------------------------------------------------
BOOL CSprite::RestoreSurfaces()
{
	bool result=true;

	//e Surface 
	for (int i=0; i<FrameNum; i++)
	if(SUCCEEDED(lpSpriteSurfaces[i]->Restore()))
		result=result&&ImgSpritePtr->draw2Surface(lpSpriteSurfaces[i], FrameW,FrameH, 
			ImgStartX+i*(FrameW+1), ImgStartY);
		//result=result&&ImgSpritePtr->AlphaDraw2Surface(lpSpriteSurfaces[i], FrameW,FrameH, 
		//	ImgStartX+i*(FrameW+1), ImgStartY,128);
	else return FALSE;

	return result;
}

//Ҧtm Surfaces Ŷ---------------------------------------------------
void CSprite::ReleaseSurfaces()
{
	if(lpSpriteSurfaces)
	{
	  for (int i=0; i<FrameNum; i++)
	  if(lpSpriteSurfaces[i]) 
	  {
		  lpSpriteSurfaces[i]->Release();	  lpSpriteSurfaces[i]=NULL;
	  }

	  delete[] lpSpriteSurfaces; lpSpriteSurfaces=NULL;
	}		
}


int CSprite::FrameWidth()
{
	return FrameW;
}


int CSprite::FrameHeight()
{
	return FrameH;
}

int CSprite::TotalFrames()
{
	return FrameNum;
}



// For Debug Usage------------------------------------------------------------
LPDIRECTDRAWSURFACE CSprite::Frame(int i)
{
	if(i>=FrameNum) return NULL;
	return lpSpriteSurfaces[i];
}







//*******************************************************************************
// CAlphaSprite O
//*********************************************************************************

// Constructor------------------------------------------------------------------------
CAlphaSprite::CAlphaSprite(BYTE alpha) : CSprite()
{
  //~өҦb CSprite() constructorҦlʧ@C
	
  Alpha=alpha;
}

//eҦϧθƨ Surfaces W---------------------------------------------------
BOOL CAlphaSprite::RestoreSurfaces()
{
	bool result=true;

	//e Surface 
	for (int i=0; i<FrameNum; i++)
	if(SUCCEEDED(lpSpriteSurfaces[i]->Restore()))
		result=result&&ImgSpritePtr->AlphaDraw2Surface(lpSpriteSurfaces[i], FrameW,FrameH, 
			ImgStartX+i*(FrameW+1), ImgStartY,Alpha);
	else return FALSE;

	return result;
}


//] Alpha ƭ---------------------------------------------------------------
void CAlphaSprite::SetAlpha(BYTE alpha)
{
	Alpha=alpha;
	RestoreSurfaces();
}
